@@ -9,7 +9,7 @@ use heapsize::{self, HeapSizeOf};
 use index::{ByteIndex, ByteOffset, ColumnIndex, LineIndex, LineOffset, RawIndex, RawOffset};
 use span::ByteSpan;
 
-#[derive(Clone, Debug, Eq, PartialEq, Ord, PartialOrd)]
+#[derive(Clone, Debug, Eq, PartialEq, Ord, PartialOrd, Hash)]
 #[cfg_attr(feature = "serialization", derive(Deserialize, Serialize))]
 pub enum FileName {
     /// A real file on disk
@@ -24,6 +24,25 @@ impl From<PathBuf> for FileName {
     }
 }
 
+impl From<FileName> for PathBuf {
+    fn from(name: FileName) -> PathBuf {
+        match name {
+            FileName::Real(path) => path,
+            FileName::Virtual(Cow::Owned(owned)) => PathBuf::from(owned),
+            FileName::Virtual(Cow::Borrowed(borrowed)) => PathBuf::from(borrowed),
+        }
+    }
+}
+
+impl<'a> From<&'a FileName> for &'a Path {
+    fn from(name: &'a FileName) -> &'a Path {
+        match *name {
+            FileName::Real(ref path) => path,
+            FileName::Virtual(ref cow) => Path::new(cow.as_ref()),
+        }
+    }
+}
+
 impl<'a> From<&'a Path> for FileName {
     fn from(name: &Path) -> FileName {
         FileName::real(name)
@@ -42,6 +61,27 @@ impl From<&'static str> for FileName {
     }
 }
 
+impl AsRef<Path> for FileName {
+    fn as_ref(&self) -> &Path {
+        match *self {
+            FileName::Real(ref path) => path.as_ref(),
+            FileName::Virtual(ref cow) => Path::new(cow.as_ref()),
+        }
+    }
+}
+
+impl PartialEq<Path> for FileName {
+    fn eq(&self, other: &Path) -> bool {
+        self.as_ref() == other
+    }
+}
+
+impl PartialEq<PathBuf> for FileName {
+    fn eq(&self, other: &PathBuf) -> bool {
+        self.as_ref() == other.as_path()
+    }
+}
+
 impl FileName {
     pub fn real<T: Into<PathBuf>>(name: T) -> FileName {
         FileName::Real(name.into())
@@ -78,7 +118,7 @@ impl HeapSizeOf for FileName {
                 path.to_str()
                     .map(|s| unsafe { heapsize::heap_size_of(s.as_ptr()) })
                     .unwrap_or(0)
-            }
+            },
         }
     }
 }
@@ -91,9 +131,15 @@ pub enum LineIndexError {
 
 #[derive(Debug, Fail, PartialEq)]
 pub enum ByteIndexError {
-    #[fail(display = "Byte index out of bounds - given: {}, span: {}", given, span)]
+    #[fail(
+        display = "Byte index out of bounds - given: {}, span: {}",
+        given, span
+    )]
     OutOfBounds { given: ByteIndex, span: ByteSpan },
-    #[fail(display = "Byte index points within a character boundary - given: {}", given)]
+    #[fail(
+        display = "Byte index points within a character boundary - given: {}",
+        given
+    )]
     InvalidCharBoundary { given: ByteIndex },
 }
 
@@ -386,24 +432,20 @@ mod tests {
     #[test]
     fn offset() {
         let test_data = TestData::new();
-        assert!(
-            test_data
-                .filemap
-                .offset(
-                    (test_data.lines.len() as u32 - 1).into(),
-                    (test_data.lines.last().unwrap().len() as u32).into()
-                )
-                .is_ok()
-        );
-        assert!(
-            test_data
-                .filemap
-                .offset(
-                    (test_data.lines.len() as u32 - 1).into(),
-                    (test_data.lines.last().unwrap().len() as u32 + 1).into()
-                )
-                .is_err()
-        );
+        assert!(test_data
+            .filemap
+            .offset(
+                (test_data.lines.len() as u32 - 1).into(),
+                (test_data.lines.last().unwrap().len() as u32).into()
+            )
+            .is_ok());
+        assert!(test_data
+            .filemap
+            .offset(
+                (test_data.lines.len() as u32 - 1).into(),
+                (test_data.lines.last().unwrap().len() as u32 + 1).into()
+            )
+            .is_err());
     }
 
     #[test]
